public class DDMin {
	
	
	
	// Requires : s is non-null and chunk < nrChunks and nrChunks <= s.length()
	// Ensures : Gives a string such that if we divide the input string in n chunks
    //           the output is the input string with the chunk with index 
	//           chunk removed
	private static String removeChuck(String s,int nrChucks, int chunk){
		int chunkLength = s.length() / nrChucks;
		int start = chunk * chunkLength;
		int end = min (s.length(), start + chunkLength);
		return s.substring(0, start).concat(s.substring(end));
	}
	
	
	
	// Requires: t.test(input) == false and 
	//           input is non-null and
	//            nrChunks <= input.length
	// Ensures: A subsequence res of input, such 
	//          that t.test(res) == false and res is 1-minimal
	private static String ddMin(Test t, String input, int nrChucks){
		for(int i = 0 ; i < nrChucks ; i++){
			String withoutChunk = removeChuck(input, nrChucks, i);
			if(!t.test(withoutChunk)){
				 return ddMin(t,withoutChunk,max(nrChucks-1,2));
			}
		}
		if(nrChucks == input.length()) return input; 
		return ddMin(t,input, min(2*nrChucks, input.length()));
	}
	
	public static String ddMin(Test t, String input){ return ddMin(t,input,min(input.length(),2));}

	
	// Requires: prop.hasProperty(input) == true and 
	//           input is non-null and
	//            nrChunks <= input.length
	// Ensures: (Leftmost) Smallest subsequence res of input, such 
	//          that prop.hasProperty(res) == true
	private static String ddMinGlobal(Test t, String input, int nrChucks){
		String smaller = null;
		for(int i = 0 ; i < nrChucks ; i++){
			String withoutChunk = removeChuck(input, nrChucks, i);
			if(!t.test(withoutChunk)){
				 String res = ddMin(t,withoutChunk,max(nrChucks-1,2));
				 if(smaller == null || smaller.length() > res.length()){
					 smaller = res;
				 }
			}
		}
		if(smaller != null) return smaller;
		if(nrChucks == input.length()) return input; 
		return ddMin(t,input, min(2*nrChucks, input.length()));
	}

	
	
	
	
	// Requires: prop.hasProperty(input) == true and input is non-null
    // Ensures: (Leftmost) Smallest subsequence res of input, such that prop.hasProperty(res) == true
	public static String ddMinGlobal(Test t, String input) {
		if(t.test(input)){
			throw new Error("Not a failing test case");
		}
		return ddMinGlobal(t,input,min(input.length(),2));
	}


	

	
	
	
	private static int min(int a, int b) {
		return a <= b ? a : b;
	}
	
	private static int max(int a, int b) {
		return a <= b ? b : a;
	}

	
}
